@@ -35,9 +35,9 @@ class NodesNotFound(ReservationFailed):
 
     def __init__(self, resource_class, conductor_group):
         message = "No available nodes%(rc)s found%(cg)s" % {
-            'rc': 'with resource class %s' % resource_class
+            'rc': ' with resource class %s' % resource_class
             if resource_class else '',
-            'cg': 'in conductor group %s' % (conductor_group or '<default>')
+            'cg': ' in conductor group %s' % (conductor_group or '<default>')
             if conductor_group is not None else ''
         }
         self.requested_resource_class = resource_class
@@ -68,10 +68,7 @@ class CapabilitiesNotFound(ReservationFailed):
 
 
 class TraitsNotFound(ReservationFailed):
-    """Requested traits do not match any nodes.
-
-    :ivar requested_traits: Requested node's traits.
-    """
+    """DEPRECATED."""
 
     def __init__(self, message, traits):
         self.requested_traits = traits
@@ -83,10 +80,7 @@ class ValidationFailed(ReservationFailed):
 
 
 class NoNodesReserved(ReservationFailed):
-    """All nodes are already reserved or failed validation.
-
-    :ivar nodes: List of nodes that were checked.
-    """
+    """DEPRECATED."""
 
     def __init__(self, nodes):
         self.nodes = nodes
@@ -112,11 +106,12 @@ class InvalidNode(Error):
 
 
 class DeploymentFailure(Error):
-    """One or more nodes have failed the deployment.
-
-    :ivar nodes: List of failed nodes.
-    """
+    """DEPRECATED."""
 
     def __init__(self, message, nodes):
         self.nodes = nodes
         super(DeploymentFailure, self).__init__(message)
+
+
+class InvalidInstance(Error):
+    """The node(s) does not have a metalsmith instance associated."""
